# This file is part of django-4store.
#
# Created by Martin Galpin (m@66laps.com)
#
# Copyright (C) 2010 66laps Limited.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from fourstore.server import start_4store_server, stop_4store_server
from fourstore.utils import get_rdf_files

import time
import signal
import sys

HELP_MESSAGE = \
"""Creates and starts a 4store backend and HTTP server.

Importing data will overwrite existing models."""


def signal_handler(signal, frame):
    print 'Shutting down 4store servers!'
    stop_4store_server(settings.FOURSTORE_KBNAME, settings.FOURSTORE_PORT)
    sys.exit(0)

class Command(BaseCommand):
    args = "<file.n3 file.rdf dir dir file.n3...>"
    help = HELP_MESSAGE

    def handle(self, *args, **options):
        try:
            files = get_rdf_files(args)
            if files:
                print "Importing the following files: %s." % files
            start_4store_server(settings.FOURSTORE_KBNAME, settings.FOURSTORE_PORT, files)
            print "4store server is running at http://localhost:%d/" % settings.FOURSTORE_PORT
            print "Quit the server with CONTROL-C."
            signal.signal(signal.SIGINT, signal_handler)
            while 1:
                time.sleep(5000)
                continue
        except AttributeError:
            raise CommandError("You must set FOURSTORE_KBNAME and FOURSTORE_PORT in your settings.py.")



import os

DEBUG = TEMPLATE_DEBUG = True
DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = '/tmp/shorturls.db'
INSTALLED_APPS = ['fourstore']
ROOT_URLCONF = ['fourstore.urls']
TEMPLATE_DIRS = os.path.join(os.path.dirname(__file__), 'tests', 'templates')

FOURSTORE_KBNAME = "django_4store"
FOURSTORE_PORT = 6667
SPARQL_ENDPOINT = "http://localhost:6667"



# This file is part of django-4store.
#
# Created by Martin Galpin (m@66laps.com)
#
# Copyright (C) 2010 66laps Limited.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from os import listdir
from os.path import join, splitext, isdir

# Valid RDF file extensions (supported by 4s-import)
RDF_EXTENSIONS = (".rdf", ".n3")

def get_rdf_files(paths):
    """
    Get all the path of all valid RDF files in the specified list. If a
    paths is a directory, all RDF files within this directory will be
    returned.
    """
    all_files = set()
    for path in paths:
        if isdir(path):
            files = [join(path, f) for f in listdir(path)
                     if splitext(join(path, f))[1] in RDF_EXTENSIONS]
            all_files.update(set(files))
        else:
            all_files.add(path)
    return list(all_files)



# Created by Martin Galpin (m@66laps.com)
#
# Copyright (C) 2010 66laps Limited.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

import httplib
import os

from django.test import TestCase
from django.conf import settings

from rdflib import URIRef
from HTTP4Store import HTTP4Store

from fourstore.test import Base4StoreTest
from fourstore.utils import get_rdf_files
from fourstore.fixtures import find_first_fixture

# This app label
APP_LABEL = "fourstore"

# Basic SPARQL query based on the test fixtures
TEST_SPARQL = """
PREFIX foaf: <http://xmlns.com/foaf/0.1/>
SELECT ?who WHERE { ?who a foaf:Person . }
"""

# Expected response for the above query
TEST_RESPONSE = [{u'who': URIRef('http://www.example.org/people/Melissa_Robinson')},
                 {u'who': URIRef('http://www.example.org/people/Ace_Ventura')}]


class FourstoreTests(Base4StoreTest):
    kbfixtures = ["ace.n3", "melissa.rdf"]

    def setUp(self):
        Base4StoreTest.setUp(self)

        # test the HTTP server is running
        self.host = "localhost:%d" % self.port
        self.assertEqual(200, crude_http_request(self.host, "/test/"))

        self.store = HTTP4Store(settings.SPARQL_ENDPOINT)

    def tearDown(self):
        Base4StoreTest.tearDown(self)

        # test the HTTP server is no longer running
        self.assertRaises(Exception, crude_http_request, self.host, "/test/")

    def test_sparql_query(self):
        # test a SPARQL query based against our test fixtures
        self.assertEqual(TEST_RESPONSE, self.store.sparql(TEST_SPARQL))

class ManagementCommandTests(TestCase):
    def test_get_files(self):
        expected = [find_first_fixture("ace.n3", APP_LABEL)]
        self.assertEquals(expected, get_rdf_files(expected))

        expected = [find_first_fixture("melissa.rdf", APP_LABEL)]
        self.assertEquals(expected, get_rdf_files(expected))

        fixtures_dir = os.path.dirname(expected[0])
        all_files = get_rdf_files([fixtures_dir])
        self.assertTrue(find_first_fixture("ace.n3", APP_LABEL) in all_files)
        self.assertTrue(find_first_fixture("melissa.rdf", APP_LABEL) in all_files)

class FixtureTests(TestCase):
    def test_find_first_fixture(self):
        self.assertNotEqual(None, find_first_fixture("ace.n3", APP_LABEL))
        self.assertEqual(None, find_first_fixture("truman.n3", APP_LABEL))
        self.assertNotEqual(None, find_first_fixture("melissa.rdf"))

def crude_http_request(host, path, method="GET"):
    connection = httplib.HTTPConnection(host)
    connection.request(method, path)
    response = connection.getresponse()
    return response.status



# This file is part of django-4store.
#
# Created by Martin Galpin (m@66laps.com)
#
# Copyright (C) 2010 66laps Limited.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from subprocess import call, check_call

def start_4store_server(name, port, files=[]):
    """
    Start a new instance of 4store, imports any specified files
    and runs the 4s-httpd server.
    """
    check_call(["4s-backend-setup", name])
    check_call(["4s-backend", name])
    if files:
        check_call(["4s-import", name] + list(files))
    check_call(["4s-httpd", "-p", str(port), name])

def stop_4store_server(name, port):
    """
    Stops any running instances of 4s-backend and 4s-httpd.
    """
    call(["pkill", "-f", "4s-backend %s" % name])
    call(["pkill", "-f", "4s-httpd -p %d" % port])




# This file is part of django-4store.
#
# Created by Martin Galpin (m@66laps.com)
#
# Copyright (C) 2010 66laps Limited.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

import os

from django.db.models import get_apps, get_app

def find_first_fixture(filename, app_label=None):
    """
    Find the first fixture that matches a specified filename
    and within an optional installed app.
    """
    if not app_label:
        app_module_paths = get_fixture_paths()
    else:
        app_module_paths = get_app_fixture_paths(get_app(app_label))

    for path in app_module_paths:
        filepath = os.path.join(path, filename)
        if os.path.exists(filepath):
            return filepath

def get_fixture_paths():
    """Get the fixture path for all installed apps."""
    app_fixture_paths = []
    for app in get_apps():
        app_fixture_paths.extend(get_app_fixture_paths(app))
    return app_fixture_paths

def get_app_fixture_paths(app):
    """Get the fixture path for a specified app."""
    # originates in django/core/models/loading.py
    app_module_paths = []
    if hasattr(app, '__path__'):
        #It's a 'models/' subpackage
        for path in app.__path__:
            app_module_paths.append(path)
    else:
        # It's a models.py module
        app_module_paths.append(os.path.dirname(app.__file__))
    return [os.path.join(path, "fixtures") for path in app_module_paths]




import os

from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "django-4store",
    version = "0.1",
    url = "http://github.com/66laps/django-fourstore",
    license = "LGPL v3",
    description = "A small Django application that makes developing with the 4Store RDF database easier.",
    long_description = read('README.rst'),
    author = "Martin Galpin",
    author_email = "m@66laps.com",
    packages = find_packages("src"),
    package_dir = { "": "src" },
    package_data = { "": ["LICENSE", "*.rst"] },
    install_requires = ["setuptools"],

    classifiers = [
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        ]
    )



# Created by Martin Galpin (m@66laps.com)
#
# Copyright (C) 2010 66laps Limited.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from subprocess import CalledProcessError

from django.test import TestCase
from django.conf import settings

from fourstore.server import start_4store_server, stop_4store_server
from fourstore.fixtures import find_first_fixture

class Base4StoreTest(TestCase):
    '''
    A base test case that automatically creates a 4store
    backend, imports the specified test fixtures and starts
    a SPARQL HTTP server. Essentially, it creates an isolated
    4store environment for each test case and the Django
    setting.SPARQL_ENDPOINT will automatically be set.
    '''
    kbname = "django_4store_test"
    port = 6666
    kbfixtures = []

    def setUp(self):
	try:
            files = [find_first_fixture(f) for f in self.kbfixtures]
            start_4store_server(self.kbname, self.port, files)
            settings.SPARQL_ENDPOINT = "http://localhost:%d/" % self.port
        except CalledProcessError:
	    self.fail('Unable to create temporary 4store')

    def tearDown(self):
        stop_4store_server(self.kbname, self.port)
